From d570a202d29b0272e00bda6780f7a14c7c98b0ed Mon Sep 17 00:00:00 2001
From: Konstantin Kuzov <master.nosferatu@gmail.com>
Date: Fri, 31 Aug 2018 10:15:51 +0300
Subject: [PATCH 3/4] resolveip: ability to resolve to only one random ip

Signed-off-by: Konstantin Kuzov <master.nosferatu@gmail.com>
---
 .../network/utils/resolveip/src/resolveip.c   | 80 ++++++++++++++++---
 1 file changed, 67 insertions(+), 13 deletions(-)

diff --git a/package/network/utils/resolveip/src/resolveip.c b/package/network/utils/resolveip/src/resolveip.c
index f7a918d0a2..9dc51dbe95 100644
--- a/package/network/utils/resolveip/src/resolveip.c
+++ b/package/network/utils/resolveip/src/resolveip.c
@@ -15,6 +15,7 @@
 #include <stdlib.h>
 #include <unistd.h>
 #include <signal.h>
+#include <time.h>
 
 
 static void abort_query(int sig)
@@ -25,16 +26,39 @@ static void abort_query(int sig)
 static void show_usage(void)
 {
 	printf("Usage:\n");
-	printf("	resolveip -h\n");
-	printf("	resolveip [-t timeout] hostname\n");
-	printf("	resolveip -4 [-t timeout] hostname\n");
-	printf("	resolveip -6 [-t timeout] hostname\n");
+	printf("	resolveip [options] hostname\n\n");
+	printf("Options:\n");
+	printf("	-t timeout	resolve timeout\n");
+	printf("	-1		return only one random result\n");
+	printf("	-4		only IPv4 addresses\n");
+	printf("	-6		only IPv6 addresses\n");
 	exit(255);
 }
 
+unsigned int rand_interval(unsigned int min, unsigned int max)
+{
+	int r;
+	const unsigned int range = 1 + max - min;
+	const unsigned int buckets = RAND_MAX / range;
+	const unsigned int limit = buckets * range;
+
+	/* Create equal size buckets all in a row, then fire randomly towards
+	 * the buckets until you land in one of them. All buckets are equally
+	 * likely. If you land off the end of the line of buckets, try again. */
+	do
+	{
+		r = rand();
+	} while (r >= limit);
+
+	return min + (r / buckets);
+}
+
 int main(int argc, char **argv)
 {
 	int timeout = 3;
+	int only_one = 0;
+	int cnt = 0;
+	int randn;
 	int opt;
 	char ipaddr[INET6_ADDRSTRLEN];
 	void *addr;
@@ -47,10 +71,13 @@ int main(int argc, char **argv)
 		.ai_flags    = 0
 	};
 
-	while ((opt = getopt(argc, argv, "46t:h")) > -1)
+	while ((opt = getopt(argc, argv, "146t:h")) > -1)
 	{
 		switch ((char)opt)
 		{
+			case '1':
+				only_one = 1;
+				break;
 			case '4':
 				hints.ai_family = AF_INET;
 				break;
@@ -80,17 +107,44 @@ int main(int argc, char **argv)
 	if (getaddrinfo(argv[optind], NULL, &hints, &res))
 		exit(2);
 
-	for (rp = res; rp != NULL; rp = rp->ai_next)
+	if (only_one)
 	{
-		addr = (rp->ai_family == AF_INET)
-			? (void *)&((struct sockaddr_in *)rp->ai_addr)->sin_addr
-			: (void *)&((struct sockaddr_in6 *)rp->ai_addr)->sin6_addr
-		;
+		srand(time(NULL));
+
+		for (rp = res; rp != NULL; rp = rp->ai_next)
+			cnt++;
+
+		randn = rand_interval(1, cnt);
+		for (rp = res; rp != NULL && cnt >= randn ; rp = rp->ai_next, cnt--)
+		{
+			if (cnt != randn)
+				continue;
 
-		if (!inet_ntop(rp->ai_family, addr, ipaddr, INET6_ADDRSTRLEN - 1))
-			exit(3);
+			addr = (rp->ai_family == AF_INET)
+				? (void *)&((struct sockaddr_in *)rp->ai_addr)->sin_addr
+				: (void *)&((struct sockaddr_in6 *)rp->ai_addr)->sin6_addr
+			;
 
-		printf("%s\n", ipaddr);
+			if (!inet_ntop(rp->ai_family, addr, ipaddr, INET6_ADDRSTRLEN - 1))
+				exit(3);
+
+			printf("%s\n", ipaddr);
+		}
+	}
+	else
+	{
+		for (rp = res; rp != NULL; rp = rp->ai_next)
+		{
+			addr = (rp->ai_family == AF_INET)
+				? (void *)&((struct sockaddr_in *)rp->ai_addr)->sin_addr
+				: (void *)&((struct sockaddr_in6 *)rp->ai_addr)->sin6_addr
+			;
+
+			if (!inet_ntop(rp->ai_family, addr, ipaddr, INET6_ADDRSTRLEN - 1))
+				exit(3);
+
+			printf("%s\n", ipaddr);
+		}
 	}
 
 	freeaddrinfo(res);
-- 
2.17.0

